// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/kubernetes/1.0/policies.yaml"] = `
tosca_definitions_version: tosca_simple_yaml_1_3

# https://istio.io/docs/reference/config/istio.networking.v1alpha3.html

policy_types:

  TrafficPolicy:
    description: >-
      DestinationRule (with TrafficPolicy) 
`
}
