// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/kubernetes/1.0/js/workflow.js"] = `

// TODO

https://github.com/pressly/sup

https://github.com/cosiner/socker
`
}
