// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/hot/1.0/js/functions/or.js"] = `

// [https://docs.openstack.org/heat/stein/template_guide/hot_spec.html#or]

function evaluate() {
	return 'TODO';
}`
}
