// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/bpmn/1.0/profile.yaml"] = `
tosca_definitions_version: tosca_simple_yaml_1_3

metadata:

  puccini.scriptlet.import:bpmn.generate: js/generate.js

namespace: bpmn

imports:

- policies.yaml
`
}
