// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/kubernetes/1.0/js/plugins/istio.js"] = `

plugin = {
  name: 'Istio',
  process: function(clout, v, kubernetes) {
    return kubernetes
  }
};
`
}
