// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/simple-for-nfv/1.0/profile.yaml"] = `
tosca_definitions_version: tosca_simple_yaml_1_2

imports:
- nodes.yaml
`
}
