// This file was auto-generated from a YAML file

package v1_0

func init() {
	Profile["/tosca/kubernetes/1.0/artifacts.yaml"] = `
tosca_definitions_version: tosca_simple_yaml_1_3

artifact_types:

  Spec:
    description: >-
      Object specification
    file_ext: [ yaml, yml, json ]
`
}
