// This file was auto-generated from a YAML file

package v5_0_5

func init() {
	Profile["/cloudify/5.0.5/data.yaml"] = `
tosca_definitions_version: cloudify_dsl_1_3

data_types:

  boolean: {}
  integer: {}
  float: {}
  string: {}
  list: {}
  dict: {}
`
}
